@@ -2,7 +2,6 @@
 # this repository contains the full copyright notices and license terms.
 import gobject
 import gtk
-import pango
 import sys
 import json
 import locale
@@ -12,7 +11,6 @@ from collections import defaultdict
 
 from tryton.config import CONFIG
 from tryton.common.cellrendererbutton import CellRendererButton
-from tryton.common.cellrenderertoggle import CellRendererToggle
 from tryton.gui.window import Window
 from tryton.common.popup_menu import populate
 from tryton.common import RPCExecute, RPCException, node_attributes, Tooltips
@@ -261,15 +259,18 @@ class ViewTree(View):
         self.children_field = children_field
         self.sum_widgets = []
         self.sum_box = gtk.HBox()
-        self.reload = False
-        if self.attributes.get('editable'):
+        if self.attributes.get('editable') and not screen.readonly:
             self.treeview = EditableTreeView(self.attributes['editable'], self)
+            grid_lines = gtk.TREE_VIEW_GRID_LINES_BOTH
         else:
             self.treeview = TreeView(self)
+            grid_lines = gtk.TREE_VIEW_GRID_LINES_VERTICAL
+        self.mnemonic_widget = self.treeview
 
         self.parse(xml)
 
         self.treeview.set_property('rules-hint', True)
+        self.treeview.set_property('enable-grid-lines', grid_lines)
         self.treeview.set_fixed_height_mode(
             all(c.get_sizing() == gtk.TREE_VIEW_COLUMN_FIXED
                 for c in self.treeview.get_columns()))
@@ -288,7 +289,7 @@ class ViewTree(View):
         self.set_drag_and_drop()
 
         self.widget = gtk.VBox()
-        scroll = gtk.ScrolledWindow()
+        self.scroll = scroll = gtk.ScrolledWindow()
         scroll.add(self.treeview)
         scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         scroll.set_placement(gtk.CORNER_TOP_LEFT)
@@ -340,7 +341,7 @@ class ViewTree(View):
         widget = Widget(self, node_attrs)
         self.widgets[name].append(widget)
 
-        column = gtk.TreeViewColumn(field.attrs['string'])
+        column = gtk.TreeViewColumn(node_attrs['string'])
         column._type = 'field'
         column.name = name
 
@@ -374,7 +375,7 @@ class ViewTree(View):
             column.set_cell_data_func(suffix.renderer,
                 suffix.setter)
 
-        self.set_column_widget(column, field, node_attrs)
+        self.set_column_widget(column, field, node_attrs, align=widget.align)
         self.set_column_width(column, field, node_attrs)
 
         if (not self.attributes.get('sequence')
@@ -436,20 +437,14 @@ class ViewTree(View):
     def get_widget(cls, name):
         return cls.WIDGETS[name]
 
-    def set_column_widget(self, column, field, attributes, arrow=True):
+    def set_column_widget(self, column, field, attributes,
+            arrow=True, align=0.5):
         hbox = gtk.HBox(False, 2)
         label = gtk.Label(attributes['string'])
         if field and self.editable:
             required = field.attrs.get('required')
             readonly = field.attrs.get('readonly')
-            if (required or not readonly) and hasattr(pango, 'AttrWeight'):
-                # FIXME when Pango.attr_weight_new is introspectable
-                attrlist = pango.AttrList()
-                if required:
-                    attrlist.insert(pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1))
-                if not readonly:
-                    attrlist.change(pango.AttrStyle(pango.STYLE_ITALIC, 0, -1))
-                label.set_attributes(attrlist)
+            common.apply_label_attributes(label, readonly, required)
         label.show()
         help = None
         if field and field.attrs.get('help'):
@@ -470,7 +465,7 @@ class ViewTree(View):
             column.set_clickable(True)
         hbox.show()
         column.set_widget(hbox)
-        column.set_alignment(0.5)
+        column.set_alignment(align)
 
     def set_column_width(self, column, field, attributes):
         default_width = {
@@ -507,7 +502,7 @@ class ViewTree(View):
         expand = attributes.get('expand', False)
         column.set_expand(expand)
         column.set_resizable(True)
-        if not field or field.attrs['type'] != 'text':
+        if attributes.get('widget') != 'text':
             column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
     def get_column_widget(self, column):
@@ -546,7 +541,7 @@ class ViewTree(View):
                 column.arrow.set(gtk.ARROW_NONE, gtk.SHADOW_NONE)
         model = self.treeview.get_model()
         unsaved_records = [x for x in model.group if x.id < 0]
-        search_string = self.screen.screen_container.get_text() or u''
+        search_string = self.screen.screen_container.get_text() or ''
         if (self.screen.search_count == len(model)
                 or unsaved_records
                 or self.screen.parent):
@@ -613,7 +608,8 @@ class ViewTree(View):
 
     @property
     def editable(self):
-        return bool(getattr(self.treeview, 'editable', False))
+        return (bool(getattr(self.treeview, 'editable', False))
+            and not self.screen.readonly)
 
     def get_fields(self):
         return [col.name for col in self.treeview.get_columns() if col.name]
@@ -637,6 +633,10 @@ class ViewTree(View):
 
     def test_expand_row(self, widget, iter_, path):
         model = widget.get_model()
+        if model.iter_n_children(iter_) > CONFIG['client.limit']:
+            self.screen.current_record = model.get_value(iter_, 0)
+            self.screen.switch_view('form')
+            return True
         iter_ = model.iter_children(iter_)
         if not iter_:
             return False
@@ -656,25 +656,10 @@ class ViewTree(View):
         for clipboard_type in (gtk.gdk.SELECTION_CLIPBOARD,
                 gtk.gdk.SELECTION_PRIMARY):
             clipboard = self.treeview.get_clipboard(clipboard_type)
-            targets = [
-                ('STRING', 0, 0),
-                ('TEXT', 0, 1),
-                ('COMPOUND_TEXT', 0, 2),
-                ('UTF8_STRING', 0, 3)
-            ]
             selection = self.treeview.get_selection()
-            # Set to clipboard directly if not too much selected rows
-            # to speed up paste
-            # Don't use set_with_data on mac see:
-            # http://bugzilla.gnome.org/show_bug.cgi?id=508601
-            if selection.count_selected_rows() < 100 \
-                    or sys.platform == 'darwin':
-                data = []
-                selection.selected_foreach(self.copy_foreach, data)
-                clipboard.set_text('\n'.join(data), -1)
-            else:
-                clipboard.set_with_data(targets, self.copy_get_func,
-                        self.copy_clear_func, selection)
+            data = []
+            selection.selected_foreach(self.copy_foreach, data)
+            clipboard.set_text('\n'.join(data), -1)
 
     def copy_foreach(self, treemodel, path, iter, data):
         record = treemodel.get_value(iter, 0)
@@ -689,17 +674,6 @@ class ViewTree(View):
         data.append('\t'.join(values))
         return
 
-    def copy_get_func(self, clipboard, selectiondata, info, selection):
-        data = []
-        selection.selected_foreach(self.copy_foreach, data)
-        clipboard.set_text('\n'.join(data), -1)
-        del data
-        return
-
-    def copy_clear_func(self, clipboard, selection):
-        del selection
-        return
-
     def on_paste(self):
         if not self.editable:
             return
@@ -767,8 +741,7 @@ class ViewTree(View):
         treeselection.selected_foreach(_func_sel_get, data)
         if not data:
             return
-        data = str(data[0])
-        selection.set(selection.get_target(), 8, data)
+        selection.set(selection.get_target(), 8, data[0].encode('utf-8'))
         return True
 
     def drag_data_received(self, treeview, context, x, y, selection,
@@ -786,6 +759,7 @@ class ViewTree(View):
             selection_data = selection.get_data()
         if not selection_data:
             return
+        selection_data = selection_data.decode('utf-8')
 
         # Don't received if the treeview was editing because it breaks the
         # internal state of the cursor.
@@ -833,6 +807,9 @@ class ViewTree(View):
             gtk.gdk.drop_finish(context, False, etime)
         else:
             context.drop_finish(False, etime)
+        selection = self.treeview.get_selection()
+        selection.unselect_all()
+        selection.select_path(record.get_index_path(model.group))
         if self.attributes.get('sequence'):
             record.group.set_sequence(field=self.attributes['sequence'])
         return True
@@ -911,9 +888,8 @@ class ViewTree(View):
                 gobject.idle_add(pop, menu, group, record)
             return True  # Don't change the selection
         elif event.button == 2:
-            event.button = 1
-            event.state |= gtk.gdk.MOD1_MASK
-            treeview.emit('button-press-event', event)
+            with Window(allow_similar=True):
+                self.screen.row_activate()
             return True
         return False
 
@@ -950,7 +926,7 @@ class ViewTree(View):
         if last_col and last_col.name in fields:
             del fields[last_col.name]
 
-        if fields and any(fields.itervalues()):
+        if fields and any(fields.values()):
             model_name = self.screen.model_name
             try:
                 RPCExecute('model', 'ir.ui.view_tree_width', 'set_width',
@@ -1035,10 +1011,10 @@ class ViewTree(View):
     def reset(self):
         pass
 
-    def display(self):
+    def display(self, force=False):
         self.treeview.display_counter += 1
         current_record = self.screen.current_record
-        if (self.reload
+        if (force
                 or not self.treeview.get_model()
                 or (self.screen.group !=
                     self.treeview.get_model().group)):
@@ -1051,7 +1027,6 @@ class ViewTree(View):
                 selection = self.treeview.get_selection()
                 path = current_record.get_index_path(model.group)
                 selection.select_path(path)
-        self.reload = False
         if not current_record:
             selection = self.treeview.get_selection()
             selection.unselect_all()
@@ -1074,6 +1049,7 @@ class ViewTree(View):
             if not name:
                 continue
             widget = self.get_column_widget(column)
+            widget.set_editable()
             if decoder.decode(widget.attrs.get('tree_invisible', '0')):
                 column.set_visible(False)
             elif name == self.screen.exclude_field:
@@ -1126,19 +1102,19 @@ class ViewTree(View):
 
             if loaded:
                 if field.attrs['type'] == 'timedelta':
-                    converter = self.screen.context.get(
-                        field.attrs.get('converter'))
+                    converter = field.converter(self.screen.group)
                     selected_sum = common.timedelta.format(
                         selected_sum, converter)
                     sum_ = common.timedelta.format(sum_, converter)
-                elif digit:
-                    selected_sum = locale.format(
-                        '%.*f', (digit, selected_sum or 0), True)
-                    sum_ = locale.format('%.*f', (digit, sum_ or 0), True)
+                elif digit is not None:
+                    selected_sum = locale.localize(
+                        '{0:.{1}f}'.format(selected_sum or 0, digit), True)
+                    sum_ = locale.localize(
+                        '{0:.{1}f}'.format(sum_ or 0, digit), True)
                 else:
-                    selected_sum = locale.format(
-                        '%s', selected_sum or 0, True)
-                    sum_ = locale.format('%s', sum_ or 0, True)
+                    selected_sum = locale.localize(
+                        '{}'.format(selected_sum or 0), True)
+                    sum_ = locale.localize('{}'.format(sum_ or 0), True)
 
                 text = '%s / %s' % (selected_sum, sum_)
             else:
@@ -1148,7 +1124,7 @@ class ViewTree(View):
     def set_cursor(self, new=False, reset_view=True):
         self.treeview.grab_focus()
         model = self.treeview.get_model()
-        if self.screen.current_record and model:
+        if self.screen.current_record and model and self.treeview.get_realized():
             path = self.screen.current_record.get_index_path(model.group)
             if model.get_flags() & gtk.TREE_MODEL_LIST_ONLY:
                 path = (path[0],)
@@ -1170,22 +1146,10 @@ class ViewTree(View):
             records.append(model.get_value(iter_, 0))
         records = []
         sel = self.treeview.get_selection()
-        sel.selected_foreach(_func_sel_get, records)
+        if sel is not None:
+            sel.selected_foreach(_func_sel_get, records)
         return records
 
-    def unset_editable(self):
-        self.treeview.editable = False
-        for col in self.treeview.get_columns():
-            for renderer in col.get_cell_renderers():
-                if isinstance(renderer, CellRendererToggle):
-                    renderer.set_property('activatable', False)
-                elif isinstance(renderer,
-                        (gtk.CellRendererProgress, CellRendererButton,
-                            gtk.CellRendererPixbuf)):
-                    pass
-                else:
-                    renderer.set_property('editable', False)
-
     def get_selected_paths(self):
         selection = self.treeview.get_selection()
         model, rows = selection.get_selected_rows()
